@@ -24,6 +24,8 @@ from .store import FlagStore
 current_map: str = ""
 bgdata: FlagStore = FlagStore()
 mod_actors_with_life: set = set()
+added_flag_hashes: set = set()
+orphaned_flag_hashes: set = set()
 GENERATOR_FLAG_NAME_EXCEPTIONS: list = [
     "DgnMrgPrt",
 ]
@@ -101,23 +103,20 @@ def bool_flag(new_obj, old_obj, maptype: str, resettype: int = 1, revival: bool
     flag = bgdata.find("bool_data", old_hash)
     old_exists = flag.exists()
     flag = BoolFlag(revival=revival) if not old_exists else flag
-    flag.set_data_name(new_name)
-    flag.set_event_assoc(bool("LinksToObj" in new_obj))
-    flag.set_is_save(True)
-    flag.set_reset_type(resettype)
-
+    flag.data_name = new_name
+    flag.is_event_associated = bool("LinksToObj" in new_obj)
+    flag.is_save = True
+    flag.reset_type = resettype
     flag.use_name_to_override_params()
 
-    mod_flag = bgdata.find("bool_data", flag.get_hash())
+    mod_flag = bgdata.find("bool_data", flag.hash_value)
     if mod_flag.exists():
-        old_exists = True
-        old_hash = mod_flag.get_hash()
-
-    if old_exists:
-        bgdata.modify("bool_data", old_hash, flag)
+        bgdata.modify("bool_data", flag.hash_value, flag)
     else:
         bgdata.add("bool_data", flag)
 
+    added_flag_hashes.add(flag.hash_value)
+
 
 def s32_flag(new_obj, old_obj, maptype: str, resettype: int = 1, revival: bool = True) -> None:
     old_hash: int = ctypes.c_int32(zlib.crc32(get_flag_name(old_obj, maptype).encode())).value
@@ -132,66 +131,56 @@ def s32_flag(new_obj, old_obj, maptype: str, resettype: int = 1, revival: bool =
     flag = bgdata.find("s32_data", old_hash)
     old_exists = flag.exists()
     flag = S32Flag(revival=revival) if not old_exists else flag
-    flag.set_data_name(new_name)
-    flag.set_event_assoc(bool("LinksToObj" in new_obj))
-    flag.set_is_save(True)
-    flag.set_reset_type(resettype)
-
+    flag.data_name = new_name
+    flag.is_event_associated = bool("LinksToObj" in new_obj)
+    flag.is_save = True
+    flag.reset_type = resettype
     flag.use_name_to_override_params()
 
-    mod_flag = bgdata.find("s32_data", flag.get_hash())
+    mod_flag = bgdata.find("s32_data", flag.hash_value)
     if mod_flag.exists():
-        old_exists = True
-        old_hash = mod_flag.get_hash()
-
-    if old_exists:
-        bgdata.modify("s32_data", old_hash, flag)
+        bgdata.modify("s32_data", flag.hash_value, flag)
     else:
         bgdata.add("s32_data", flag)
 
+    added_flag_hashes.add(flag.hash_value)
 
-def location_flag(name: str) -> None:
+
+def misc_s32_flag(name: str) -> None:
     flag = S32Flag()
-    flag.set_data_name(name)
-    flag.set_init_value(0)
-    flag.set_max_value(2147483647)
-    flag.set_min_value(-2147483648)
-
+    flag.data_name = name
     flag.use_name_to_override_params()
 
-    old_flag = bgdata.find("s32_data", flag.get_hash())
+    old_flag = bgdata.find("s32_data", flag.hash_value)
     if old_flag.exists():
-        bgdata.modify("s32_data", old_flag.get_hash(), flag)
+        bgdata.modify("s32_data", old_flag.hash_value, flag)
     else:
         bgdata.add("s32_data", flag)
 
+    added_flag_hashes.add(flag.hash_value)
+
 
 def misc_bool_flag(name: str) -> None:
     flag = BoolFlag()
-    flag.set_data_name(name)
-
+    flag.data_name = name
     flag.use_name_to_override_params()
 
-    old_flag = bgdata.find("bool_data", flag.get_hash())
+    old_flag = bgdata.find("bool_data", flag.hash_value)
     if old_flag.exists():
-        bgdata.modify("bool_data", old_flag.get_hash(), flag)
+        bgdata.modify("bool_data", old_flag.hash_value, flag)
     else:
         bgdata.add("bool_data", flag)
 
+    added_flag_hashes.add(flag.hash_value)
+
 
 def generate_revival_flags_for_map(
     map_data: oead.byml.Hash, stock_map: oead.byml.Hash, maptype: str, resettype: int
-) -> dict:
-    r: dict = {"ignore": set(), "delete": set()}
+) -> None:
 
     map_hashes = [obj["HashId"].v for obj in map_data["Objs"]]
     stock_hashes = [obj["HashId"].v for obj in stock_map["Objs"]]
     for obj in map_data["Objs"]:
-        r["ignore"].add(
-            ctypes.c_int32(
-                zlib.crc32(f"{maptype}_{obj['UnitConfigName']}_{obj['HashId'].v}".encode())
-            ).value
-        )
         revival = True
         bflag = True
         if "LinkTag" in obj["UnitConfigName"]:
@@ -208,27 +197,20 @@ def generate_revival_flags_for_map(
                 s32_flag(obj, obj, maptype, resettype, revival)
         elif obj["HashId"].v in stock_hashes:
             stock_obj = stock_map["Objs"][stock_hashes.index(obj["HashId"].v)]
-            name_changed = obj["UnitConfigName"] != stock_obj["UnitConfigName"]
-            event_assoc_changed = bool("LinksToObj" in obj) != bool("LinksToObj" in stock_obj)
-            if name_changed or event_assoc_changed:
-                if bflag:
-                    bool_flag(obj, stock_obj, maptype, resettype, revival)
-                else:
-                    s32_flag(obj, stock_obj, maptype, resettype, revival)
+            if bflag:
+                bool_flag(obj, stock_obj, maptype, resettype, revival)
+            else:
+                s32_flag(obj, stock_obj, maptype, resettype, revival)
     for obj in stock_map["Objs"]:
         if obj["HashId"].v not in map_hashes:
             old_hash: int = ctypes.c_int32(
                 zlib.crc32(f"{maptype}_{obj['UnitConfigName']}_{obj['HashId'].v}".encode())
             ).value
-            r["delete"].add(old_hash)
-
-    return r
+            orphaned_flag_hashes.add(old_hash)
 
 
 def generate_revival_flags(resettypes: list) -> None:
     moddir: Path = util.root_dir()
-    ignore_hashes: set = set()
-    delete_hashes: set = set()
 
     if not resettypes[0] == -1:
         for map_unit in moddir.rglob("*_*.smubin"):
@@ -236,10 +218,8 @@ def generate_revival_flags(resettypes: list) -> None:
             map_data = oead.byml.from_binary(oead.yaz0.decompress(map_unit.read_bytes()))
             map_section = map_unit.stem.split("_")
             stock_map = mubin.get_stock_map((map_section[0], map_section[1]))
-            temp = generate_revival_flags_for_map(map_data, stock_map, "MainField", resettypes[0])
+            generate_revival_flags_for_map(map_data, stock_map, "MainField", resettypes[0])
             print(f"Finished processing {map_unit.name} in {time.time() - map_start} seconds...")
-            ignore_hashes |= temp["ignore"]
-            delete_hashes |= temp["delete"]
         for static_unit in moddir.rglob("MainField/Static.smubin"):
             map_start = time.time()
             static_data = oead.byml.from_binary(oead.yaz0.decompress(static_unit.read_bytes()))
@@ -250,7 +230,7 @@ def generate_revival_flags(resettypes: list) -> None:
                     continue
                 if "MessageID" in marker:
                     if not marker["MessageID"] in vanilla_shrine_locs:
-                        location_flag(marker["SaveFlag"])
+                        misc_s32_flag(marker["SaveFlag"])
                         misc_bool_flag(f"Enter_{marker['MessageID']}")
                         misc_bool_flag(f"CompleteTreasure_{marker['MessageID']}")
             print(
@@ -283,92 +263,122 @@ def generate_revival_flags(resettypes: list) -> None:
                 else:
                     stock_map = oead.byml.Hash()
                     stock_map["Objs"] = oead.byml.Array()
-                temp = generate_revival_flags_for_map(
-                    map_data, stock_map, "CDungeon", resettypes[1]
-                )
+                generate_revival_flags_for_map(map_data, stock_map, "CDungeon", resettypes[1])
                 print(f"Finished processing {map_name} in {time.time() - map_start} seconds...")
-                ignore_hashes |= temp["ignore"]
-                delete_hashes |= temp["delete"]
             current_map = ""
             # ^ unnecessary given that it's updated every loop, but we rely on it possibly being
             # empty for determining whether LinkTags are valid, so it's better to be safe
-    for hash in delete_hashes:
-        if hash not in ignore_hashes:
+    for hash in orphaned_flag_hashes:
+        if hash not in added_flag_hashes:
             bgdata.remove("bool_data", hash)
 
 
-def actor_bool_flag(flag_type: str, actor_name: str, cat: int = -1) -> int:
-    flag_name: str = f"{flag_type}_{actor_name}"
+def actor_bool_flag(flag_name: str) -> int:
     flag_hash: int = ctypes.c_int32(zlib.crc32(flag_name.encode())).value
 
     flag = bgdata.find("bool_data", flag_hash)
     old_exists = flag.exists()
     flag = BoolFlag() if not old_exists else flag
-    flag.set_data_name(flag_name)
-    if not cat == -1:
-        flag.set_category(cat)
-
+    flag.data_name = flag_name
+    flag.is_save = True
     flag.use_name_to_override_params()
 
-    mod_flag = bgdata.find("bool_data", flag.get_hash())
+    mod_flag = bgdata.find("bool_data", flag.hash_value)
     if mod_flag.exists():
-        old_exists = True
-        old_hash = mod_flag.get_hash()
-
-    if old_exists:
-        bgdata.modify("bool_data", flag_hash, flag)
+        bgdata.modify("bool_data", flag.hash_value, flag)
     else:
         bgdata.add("bool_data", flag)
 
-    return flag.get_hash()
+    return flag.hash_value
 
 
-def actor_s32_flag(flag_type: str, actor_name: str) -> int:
-    flag_name: str = f"{flag_type}_{actor_name}"
+def actor_s32_flag(flag_name: str) -> int:
     flag_hash: int = ctypes.c_int32(zlib.crc32(flag_name.encode())).value
 
     flag = bgdata.find("s32_data", flag_hash)
     old_exists = flag.exists()
     flag = S32Flag() if not old_exists else flag
-    flag.set_data_name(flag_name)
-    flag.set_max_value(2147483647)
-    flag.set_min_value(0)
-
+    flag.data_name = flag_name
+    flag.is_save = True
     flag.use_name_to_override_params()
 
-    mod_flag = bgdata.find("bool_data", flag.get_hash())
+    mod_flag = bgdata.find("bool_data", flag.hash_value)
     if mod_flag.exists():
-        old_exists = True
-        old_hash = mod_flag.get_hash()
-
-    if old_exists:
-        bgdata.modify("s32_data", flag_hash, flag)
+        bgdata.modify("s32_data", flag.hash_value, flag)
     else:
         bgdata.add("s32_data", flag)
 
-    return flag.get_hash()
+    return flag.hash_value
 
 
 def generate_item_flags() -> None:
     moddir: Path = util.root_dir()
     mod_bool: set = set()
     mod_s32: set = set()
-    for item_actor in moddir.rglob("**/Item_*.sbactorpack"):
-        mod_bool.add(actor_bool_flag("IsNewPictureBook", str(item_actor.stem)))
-        mod_bool.add(actor_bool_flag("IsRegisteredPictureBook", str(item_actor.stem), 4))
-        mod_bool.add(actor_bool_flag("IsGet", str(item_actor.stem)))
-        mod_s32.add(actor_s32_flag("PictureBookSize", str(item_actor.stem)))
-    for armor_actor in moddir.rglob("**/Armor_*.sbactorpack"):
-        mod_bool.add(actor_bool_flag("IsGet", str(armor_actor.stem)))
-        mod_s32.add(actor_s32_flag("EquipTime", str(armor_actor.stem)))
-        mod_s32.add(actor_s32_flag("PorchTime", str(armor_actor.stem)))
-    for weapon_actor in moddir.rglob("**/Weapon_*.sbactorpack"):
-        mod_bool.add(actor_bool_flag("IsNewPictureBook", str(weapon_actor.stem)))
-        mod_bool.add(actor_bool_flag("IsRegisteredPictureBook", str(weapon_actor.stem), 5))
-        mod_bool.add(actor_bool_flag("IsGet", str(weapon_actor.stem)))
-        mod_s32.add(actor_s32_flag("PictureBookSize", str(weapon_actor.stem)))
-        mod_s32.add(actor_s32_flag("EquipTime", str(weapon_actor.stem)))
-        mod_s32.add(actor_s32_flag("PorchTime", str(weapon_actor.stem)))
+    for actor in moddir.rglob("*.sbactorpack"):
+        if "Animal_" in actor.name:
+            mod_bool.add(actor_bool_flag(f"IsNewPictureBook_{actor.stem}"))
+            mod_bool.add(actor_bool_flag(f"IsRegisteredPictureBook_{actor.stem}"))
+            mod_s32.add(actor_s32_flag(f"PictureBookSize_{actor.stem}"))
+            bxml = oead.aamp.ParameterIO.from_binary(
+                oead.Sarc(oead.yaz0.decompress(actor.read_bytes()))
+                .get_file(f"Actor/ActorLink/{actor.stem}.bxml")
+                .data
+            )
+            if "Tags" in bxml.objects:
+                for _, tag in bxml.objects["Tags"].params.items():
+                    if tag == "CanGetPouch":
+                        mod_bool.add(actor_bool_flag(f"IsGet_{actor.stem}"))
+                        break
+            del bxml
+        elif "Armor_" in actor.name:
+            mod_bool.add(actor_bool_flag(f"IsGet_{actor.stem}"))
+            mod_s32.add(actor_s32_flag(f"EquipTime_{actor.stem}"))
+            mod_s32.add(actor_s32_flag(f"PorchTime_{actor.stem}"))
+        elif "Enemy_" in actor.name:
+            mod_bool.add(actor_bool_flag(f"IsNewPictureBook_{actor.stem}"))
+            mod_bool.add(actor_bool_flag(f"IsRegisteredPictureBook_{actor.stem}"))
+            mod_s32.add(actor_s32_flag(f"PictureBookSize_{actor.stem}"))
+        elif "Item_" in actor.name:
+            mod_bool.add(actor_bool_flag(f"IsNewPictureBook_{actor.stem}"))
+            mod_bool.add(actor_bool_flag(f"IsRegisteredPictureBook_{actor.stem}"))
+            mod_bool.add(actor_bool_flag(f"IsGet_{actor.stem}"))
+            mod_s32.add(actor_s32_flag(f"PictureBookSize_{actor.stem}"))
+        elif "Npc_" in actor.name:
+            sarc = oead.Sarc(oead.yaz0.decompress(actor.read_bytes()))
+            bxml = oead.aamp.ParameterIO.from_binary(
+                sarc.get_file(f"Actor/ActorLink/{actor.stem}.bxml").data
+            )
+            shop_link = bxml.objects["LinkTarget"].params["ShopDataUser"]
+            del bxml
+            if shop_link == "Dummy":
+                del sarc
+                continue
+            mod_bool.add(actor_bool_flag(f"{actor.stem}_SoldOut"))
+            bshop = oead.aamp.ParameterIO.from_binary(
+                sarc.get_file(f"Actor/ShopData/{shop_link}.bshop").data
+            )
+            del sarc
+            for tablekey, tablename in bshop.objects["Header"].params.items():
+                if tablekey.hash == zlib.crc32(b"TableNum"):
+                    continue
+                for _, val in bshop.objects[str(tablename)].params.items():
+                    if isinstance(val, oead.FixedSafeString64):
+                        mod_s32.add(actor_s32_flag(f"{actor.stem}_{str(val)}"))
+            del bshop
+        elif "Weapon_" in actor.name:
+            mod_bool.add(actor_bool_flag(f"IsNewPictureBook_{actor.stem}"))
+            mod_bool.add(actor_bool_flag(f"IsRegisteredPictureBook_{actor.stem}"))
+            mod_bool.add(actor_bool_flag(f"IsGet_{actor.stem}"))
+            mod_s32.add(actor_s32_flag(f"PictureBookSize_{actor.stem}"))
+            mod_s32.add(actor_s32_flag(f"EquipTime_{actor.stem}"))
+            mod_s32.add(actor_s32_flag(f"PorchTime_{actor.stem}"))
+
+    """
+    This code section is meant to delete flags for actors that have been removed.
+    However, the likelihood of this ever actually needing to happen is so low,
+    and the cost of maintaining it so comparatively high, that it's being removed
+    for now. I might explore reimplementing it at a later date.
 
     vanilla_hashes: set = set()
     for _, hash_list in vanilla_hash_dict.items():
@@ -389,6 +399,7 @@ def generate_item_flags() -> None:
     to_delete = total_s32 - (mod_s32 | vanilla_hashes)
     for hash in to_delete:
         bgdata.remove("s32_data", hash)
+    """
 
 
 def generate(args):
